package com.qmth.ops.biz.ai.client.azure.llm;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.qmth.boot.core.ai.model.llm.ChatRequest;
import com.qmth.boot.core.ai.model.llm.ChatResult;
import com.qmth.boot.core.ai.model.llm.ChatRole;
import com.qmth.boot.core.exception.NotFoundException;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.ops.biz.ai.client.ChatApiClient;
import com.qmth.ops.biz.ai.client.ChatApiConfig;
import com.qmth.ops.biz.ai.client.Credentials;
import com.qmth.ops.biz.ai.exception.ChatRequestError;

import java.io.IOException;
import java.nio.charset.StandardCharsets;

public class AzureChatClient extends ChatApiClient {

    private static final String AUTH_HEADER_NAME = "api-key";

    private static final String MODEL_PLACEHOLDER = "{model}";

    public AzureChatClient(ChatApiConfig config) {
        super(config);
        config.setUrl(config.getUrl().replace(MODEL_PLACEHOLDER, config.getModel()));
    }

    @Override
    protected Credentials gerCredentials() {
        return new Credentials(AUTH_HEADER_NAME, getConfig().getSecret());
    }

    @Override
    protected Object buildRequest(ChatRequest request) {
        return request;
    }

    @Override
    protected ChatResult buildResult(byte[] data, ObjectMapper mapper) throws IOException {
        return mapper.readValue(data, AzureChatResult.class).buildResult();
    }

    @Override
    protected ChatResult handleError(byte[] data, int statusCode, ObjectMapper mapper) {
        String message = data != null ? new String(data, StandardCharsets.UTF_8) : null;
        switch (statusCode) {
        case 400:
            throw new ChatRequestError(message != null ? message : "chat request error");
        case 404:
            throw new NotFoundException(message != null ? message : "chat resource not found");
        default:
            throw new StatusException(message != null ? message : "chat model error");
        }
    }

    public static void main(String[] args) throws Exception {
        ChatApiConfig config = new ChatApiConfig();
        config.setSupplier("Azure");
        config.setUrl(
                "https://qmth-ai.openai.azure.com/openai/deployments/{model}/chat/completions?api-version=2024-02-01");
        config.setSecret("ed11ce6c22984fc085db7acdd023828b");
        config.setModel("gpt35");
        config.setQpm(0);
        AzureChatClient client = new AzureChatClient(config);
        ChatRequest request = new ChatRequest();
        request.addMessage(ChatRole.user, "who are you");
        System.out.println(new ObjectMapper().writeValueAsString(client.call(request)));
    }
}
